﻿using SqlSugar;
using System;
using System.Collections.Generic;
using System.Text;



namespace SlnMesnac.Model.domain
{
    
    [SugarTable("mes_base_barcode_info"), TenantAttribute("mes")]
    public partial class MesBaseBarcodeInfo
    { 
        /// <summary>
        /// Desc:id   
        /// </summary>           
        [SugarColumn(IsPrimaryKey = true, IsIdentity = true, ColumnName = "barcode_id")]
        public long barcodeId { get; set; }

        /// <summary>
        /// 打印时间
        /// </summary>          
        [SugarColumn(ColumnName = "print_time")]
        public DateTime? printTime { get; set; }

        /// <summary>
        ///打印人 
        /// </summary>          
        [SugarColumn(ColumnName = "print_person")]
        public string? printPerson { get; set; }

        /// <summary>
        /// 打印标识(0否1是)
        /// </summary>
        [SugarColumn(ColumnName = "print_flag")]
        public string printFlag { get; set; }

        /// <summary>
        /// 批次标识(0否，1是);在打印时，如果物料已设置，则根据物料的批次信息，如果没有设置则在此需要选择，选择物料等信息保存后同步设置物料信息。
        /// </summary>
        [SugarColumn(ColumnName = "batch_flag")]
        public string batchFlag { get; set; }

        /// <summary>
        ///条码类型（1原材料,2半成品,3成品,4背板）
        /// </summary>
        [SugarColumn(ColumnName = "barcode_type")]
        public string barcodeType { get; set; }

        /// <summary>
        /// 条码内容;如果是按单个物料贴，则一个物料一个条码，如果是按批次贴，则条码跟批次代码相同
        /// </summary>
        [SugarColumn(ColumnName = "barcode_info")]
        public string barcodeInfo { get; set; }

        /// <summary>
        /// 批次代码
        /// </summary>
        [SugarColumn(ColumnName = "batch_code")]
        public string batchCode { get; set; }

        /// <summary>
        ///托盘RFID代码
        /// </summary>
        [SugarColumn(ColumnName = "pallet_info_code")]
        public string palletInfoCode { get; set; }

        /// <summary>
        /// 绑定的物料条码（用来拆分绑定使用）
        /// </summary>
        [SugarColumn(ColumnName = "bind_barcode")]
        public string? bindBarcode { get; set; }

        /// <summary>
        /// 物料ID，关联mes_base_material_info的material_id
        /// </summary>
        [SugarColumn(ColumnName = "material_id")]
        public long? materialId { get; set; }

        /// <summary>
        ///供应商ID
        /// </summary>
        [SugarColumn(ColumnName = "manufacturer_id")]
        public long? manufacturerId { get; set; }

        /// <summary>
        /// 数量;如果是批次需要输入此字段
        /// </summary>
        [SugarColumn(ColumnName = "amount")]
        public decimal? amount { get; set; }

        /// <summary>
        /// 成品条码，在四楼成品到五楼作为原材料时需要绑定之前的成品条码 
        /// </summary>
        [SugarColumn(ColumnName = "product_barcode")]
        public string? productBarcode { get; set; }

        /// <summary>
        /// 打印数量
        /// </summary>
        [SugarColumn(ColumnName = "print_number")]
        public int? printNumber { get; set; }

        /// <summary>
        ///  打印机台名称，打印时获取  
        /// </summary>
        [SugarColumn(ColumnName = "machine_name")]
        public string? machineName { get; set; }

        /// <summary>
        /// 采购订单ID
        /// </summary>
        [SugarColumn(ColumnName = "purchase_order_id")]
        public long? purchaseOrderId { get; set; }

        /// <summary>
        /// 采购订单号;适合原材料入库时
        /// </summary>
        [SugarColumn(ColumnName = "po_no")]
        public string? poNo { get; set; }

        /// <summary>
        ///生产日期;打印时根据领料单选择输入
        /// </summary>
        [SugarColumn(ColumnName = "production_date")]
        public DateTime? productionDate { get; set; }

        /// <summary>
        ///接受日期;打印时的系统日期
        /// </summary>
        [SugarColumn(ColumnName = "accepted_date")]
        public DateTime? acceptedDate { get; set; }

        /// <summary>
        /// 最晚出库日期;可以直接输入，也可以根据日限计算
        /// </summary>
        [SugarColumn(ColumnName = "last_outstock_date")]
        public DateTime? lastOutstockDate { get; set; }

        /// <summary>
        /// 生产计划编号，关联mes_pd_product_plan的plan_code;适合生产出入库等
        /// </summary>
        [SugarColumn(ColumnName = "plan_code")]
        public string planCode { get; set; }

        /// <summary>
        /// 生产计划明细编号，关联mes_product_plan_detail的plan_detail_code;适合生产出入库等
        /// </summary>
        [SugarColumn(ColumnName = "plan_detail_code")]
        public string? planDetailCode { get; set; }

        /// <summary>
        /// 安全库存标识(1是，0否)，如果销售订单是0，则此字段是安全库存
        /// </summary>
        [SugarColumn(ColumnName = "safe_flag")]
        public string safeFlag { get; set; }

        /// <summary>
        /// 销售订单ID
        /// </summary>
        [SugarColumn(ColumnName = "sale_order_id")]
        public long? saleOrderId { get; set; }

        /// <summary>
        /// 销售订单编号;适合生产出入库等
        /// </summary>
        [SugarColumn(ColumnName = "saleorder_code")]
        public string saleorderCode { get; set; }

        /// <summary>
        /// 项目号
        /// </summary>
        [SugarColumn(ColumnName = "project_no")]
        public string? projectNo { get; set; }

        /// <summary>
        /// 流水号
        /// </summary>
        [SugarColumn(ColumnName = "serial_number")]
        public string? serialNumber { get; set; }

        /// <summary>
        /// Desc:
        /// Default:
        /// Nullable:True
        /// </summary>
        [SugarColumn(ColumnName = "remark")]
        public string? remark { get; set; }

        /// <summary>
        /// 绑定状态(1绑定，0解绑)
        /// </summary>
        [SugarColumn(ColumnName = "bind_status")]
        public string? bindStatus { get; set; }

        /// <summary>
        /// Desc:
        /// Default:
        /// Nullable:True
        /// </summary>
        [SugarColumn(ColumnName = "bind_by")]
        public string? bindBy { get; set; }

        /// <summary>
        /// 绑定托盘时间
        /// </summary>
        [SugarColumn(ColumnName = "bind_time")]
        public DateTime? bindTime { get; set; }

        /// <summary>
        /// Desc:
        /// Default:
        /// Nullable:True
        /// </summary>
        [SugarColumn(ColumnName = "update_by")]
        public string? updateBy { get; set; }

        /// <summary>
        /// 更新时间
        /// </summary>
        [SugarColumn(ColumnName = "update_time")]
        public DateTime? updateTime { get; set; }

        /// <summary>
        /// Desc:
        /// Default:
        /// Nullable:True
        /// </summary>
        [SugarColumn(ColumnName = "transferred_plan_detail_code")]
        public string? transferredPlanDetailCode { get; set; }

        /// <summary>
        /// 完整表示
        /// </summary>
        [SugarColumn(ColumnName = "complete_flag")]
        public string? completeFlag { get; set; }

        /// <summary>
        /// Desc:
        /// Default:0
        /// Nullable:False
        /// </summary>
        [SugarColumn(ColumnName = "single_flag")]
        public string singleFlag { get; set; }

        /// <summary>
        /// 物料规格
        /// </summary>
        [SugarColumn(ColumnName = "barcode_spec")]
        public string barcodeSpec { get; set; }
    }

}
